package cn.iocoder.yudao.module.reward.service.reward;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQuery;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.reward.convert.reward.RewardConvert;
import cn.iocoder.yudao.module.reward.dal.dataobject.reward.RewardDO;
import cn.iocoder.yudao.module.reward.dal.mysql.reward.RewardMapper;
import cn.iocoder.yudao.module.reward.vo.reward.*;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.RandomStringUtils;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.time.Instant;
import java.util.Collection;
import java.util.Date;
import java.util.List;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.reward.enums.ErrorCodeConstants.*;

/**
 * 礼品 Service 实现类
 *
 * @author 系统管理员
 */
@Service
@Validated
public class RewardServiceImpl extends AbstractService<RewardMapper, RewardDO> implements RewardService {

    @Resource
    private RewardMapper rewardMapper;

    @Override
    public Long create(RewardCreateReqVO createReqVO) {
        //活动开始时间不能比活动结束时间晚，活动结束时间不能比当前时间早
        if (ObjectUtils.allNotNull(createReqVO.getStartTime(), createReqVO.getEndTime())) {
            if (createReqVO.getStartTime().after((createReqVO.getEndTime())) || createReqVO.getEndTime().before(Date.from(Instant.now()))) {
                throw exception(REWARD_START_OR_END_TIME_NOT_ALLOW_CREATE);
            }
        } else if (ObjectUtils.allNull(createReqVO.getStartTime(), createReqVO.getEndTime())) {
        } else {
            throw exception(REWARD_START_OR_END_TIME_NOT_ALLOW_CREATE);
        }
        // 创建的礼品状态只能是启用或未启用
        if (createReqVO.getStatus() != 1 && createReqVO.getStatus() != 2) {
            throw exception(REWARD_STATUS_NOT_ALLOW_CREATE);
        }
        //领取方式只能是1，2，3
        if (createReqVO.getPickMethod() != 1 && createReqVO.getPickMethod() != 2 && createReqVO.getPickMethod() != 3) {
            throw exception(REWARD_PICK_METHOD_NOT_ALLOW_CREATE);
        }
        // 插入
        RewardDO rewardDO = RewardConvert.INSTANCE.convert(createReqVO);
        rewardDO.setCode(generateRewardCode());
        //如果插入失败，重新生成code再次插入
        try {
            rewardMapper.insert(rewardDO);
        } catch (Exception e) {
            rewardDO.setCode(generateRewardCode());
            rewardMapper.insert(rewardDO);
        }
        // 返回
        return rewardDO.getId();
    }

    @Override
    public void update(RewardUpdateReqVO updateReqVO) {
        // 校验存在
        RewardDO rewardDO = rewardMapper.selectById(updateReqVO.getId());
        if (rewardDO == null) {
            throw exception(REWARD_NOT_EXISTS);
        }
        RewardDO updateObj = new RewardDO();
        //未启用可编辑所有内容
        if (rewardDO.getStatus() == 2) {
            if (ObjectUtils.allNotNull(updateReqVO.getStartTime(), updateReqVO.getEndTime())) {
                if (updateReqVO.getStartTime().after((updateReqVO.getEndTime())) || updateReqVO.getEndTime().before(Date.from(Instant.now()))) {
                    throw exception(REWARD_START_OR_END_TIME_NOT_ALLOW_CREATE);
                }
            } else if (ObjectUtils.allNull(updateReqVO.getStartTime(), updateReqVO.getEndTime())) {
            } else {
                throw exception(REWARD_START_OR_END_TIME_NOT_ALLOW_CREATE);
            }
            //领取方式只能是1，2，3
            if (updateReqVO.getPickMethod() != 1 && updateReqVO.getPickMethod() != 2 && updateReqVO.getPickMethod() != 3) {
                throw exception(REWARD_PICK_METHOD_NOT_ALLOW_CREATE);
            }
            updateObj = RewardConvert.INSTANCE.convert(updateReqVO);
        } else if (rewardDO.getStatus() == 1) { //已启用可编辑剩余数量和备注
            updateObj.setQuantityRemain(updateReqVO.getQuantityRemain());
            updateObj.setRemarkEn(updateReqVO.getRemarkEn());
            updateObj.setRemarkFr(updateReqVO.getRemarkFr());
            updateObj.setRemarkZh(updateReqVO.getRemarkZh());
        } else {
            throw exception(REWARD_STATUS_NOT_ALLOW_UPDATE);
        }
        // 更新
        rewardMapper.updateById(updateObj);
    }

    @Override
    public void delete(Long id) {
        RewardDO rewardDO = rewardMapper.selectById(id);
        if (rewardDO == null) {
            throw exception(REWARD_NOT_EXISTS);
        }
        //只有未启用可以删除
        if (rewardDO.getStatus() != 2) {
            throw exception(REWARD_STATUS_NOT_ALLOW_DELETE);
        }
        // 删除
        rewardMapper.deleteById(id);
    }

    private void validateExists(Long id) {
        if (rewardMapper.selectById(id) == null) {
            throw exception(REWARD_NOT_EXISTS);
        }
    }

    @Override
    public RewardDO get(Long id) {
        RewardDO rewardDO = rewardMapper.selectById(id);
        //校验是否过期
        if (rewardDO != null) {
            validateExpire(rewardDO);
        }
        return rewardDO;
    }

    @Override
    public List<RewardDO> getList(Collection<Long> ids) {
        List<RewardDO> rewardDOS = rewardMapper.selectBatchIds(ids);
        //校验是否过期
        rewardDOS.forEach(this::validateExpire);
        return rewardDOS;
    }

    @Override
    public PageResult<RewardDO> getPage(RewardQueryVO query, PageVO page) {

        LambdaQuery<RewardDO> lambdaQuery = new LambdaQuery<>();
        //礼品ID
        lambdaQuery.eqIfPresent(RewardDO::getCode, query.getCode());
        //礼品名称，模糊匹配三种语言
        if (query.getTitle() != null) {
            lambdaQuery.and(wrapper ->
                    wrapper.like(RewardDO::getTitleZh, query.getTitle())
                            .or()
                            .like(RewardDO::getTitleEn, query.getTitle())
                            .or()
                            .like(RewardDO::getTitleFr, query.getTitle())
            );
        }
        //兑换积分
        if (query.getPointsRequireSymbol() != null && query.getPointsRequire() != null) {
            lambdaQuery.gt(query.getPointsRequireSymbol() == 1, RewardDO::getPointsRequire, query.getPointsRequire())
                    .eq(query.getPointsRequireSymbol() == 2, RewardDO::getPointsRequire, query.getPointsRequire())
                    .lt(query.getPointsRequireSymbol() == 3, RewardDO::getPointsRequire, query.getPointsRequire());
        }
        //兑换网点id，领取方式，状态
        lambdaQuery.eqIfPresent(RewardDO::getNodeId, query.getNodeId())
                .eqIfPresent(RewardDO::getPickMethod, query.getPickMethod())
                .eqIfPresent(RewardDO::getStatus, query.getStatus());
        //备注，模糊匹配三种语言
        if (query.getRemark() != null) {
            lambdaQuery.and(wrapper ->
                    wrapper.like(RewardDO::getRemarkZh, query.getRemark())
                            .or()
                            .like(RewardDO::getRemarkEn, query.getRemark())
                            .or()
                            .like(RewardDO::getRemarkFr, query.getRemark())
            );
        }
        //剩余数量
        if (query.getQuantityRemainSymbol() != null && query.getQuantityRemain() != null) {
            lambdaQuery.gt(query.getQuantityRemainSymbol() == 1, RewardDO::getQuantityRemain, query.getQuantityRemain())
                    .eq(query.getQuantityRemainSymbol() == 2, RewardDO::getQuantityRemain, query.getQuantityRemain())
                    .lt(query.getQuantityRemainSymbol() == 3, RewardDO::getQuantityRemain, query.getQuantityRemain());
        }
        //创建时间
        lambdaQuery.betweenIfPresent(RewardDO::getCreateTime, query.getBeginCreateTime(), query.getEndCreateTime())
                .orderByDesc(RewardDO::getId);
        PageResult<RewardDO> rewardDOPageResult = rewardMapper.selectPage(page, lambdaQuery);
        //校验是否过期
        rewardDOPageResult.getList().forEach(this::validateExpire);
        return rewardDOPageResult;
    }

    @Override
    public List<RewardDO> getList(RewardQueryVO query) {
        return rewardMapper.selectList(query);
    }

    @Override
    public Long copyReward(Long id) {
        RewardDO reward = rewardMapper.selectById(id);
        if (reward == null) {
            throw exception(REWARD_NOT_EXISTS);
        }
        //重新生成礼品ID
        reward.setCode(generateRewardCode());
        //设置为未启用
        reward.setStatus(2);

        reward.setId(null);
        reward.setCreateTime(null);
        reward.setUpdateTime(null);
        reward.setDeleted(null);
        try {
            rewardMapper.insert(reward);
        } catch (Exception e) {
            reward.setCode(generateRewardCode());
            rewardMapper.insert(reward);
        }
        return reward.getId();
    }

    @Override
    public void delayReward(RewardDelayReqVO delayVO) {
        RewardDO reward = rewardMapper.selectById(delayVO.getId());
        if (reward == null) {
            throw exception(REWARD_NOT_EXISTS);
        }
        //只允许延期已启用状态礼品
        if (reward.getStatus() != 1) {
            throw exception(REWARD_STATUS_NOT_ALLOW_DELAY);
        }
        Instant now = Instant.now();
        //结束时间不能小于当前时间
        if (!delayVO.getEndTime().toInstant().isAfter(now)) {
            throw exception(REWARD_ENDTIME_ERROR);
        }
        rewardMapper.updateById(RewardConvert.INSTANCE.convertDelayReqVO(delayVO));
    }

    @Override
    public void updateStatus(RewardStatusReqVO statusVO) {
        this.validateExists(statusVO.getId());
        RewardDO rewardDO = rewardMapper.selectById(statusVO.getId());
        Integer oldStatus = rewardDO.getStatus();
        Integer newStatus = statusVO.getStatus();
        switch (newStatus) {
            case 3://关闭：未启用情况下，不可进行关闭；点击关闭需要有二次弹窗提示；关闭后状态为已关闭，不支持兑换
                if (oldStatus == 2) {
                    throw exception(REWARD_STATUS_NOT_ALLOW_ClOSE);
                } else {
                    rewardMapper.updateById(RewardConvert.INSTANCE.convertStatusReqVO(statusVO));
                }
                break;
            case 1://启用：有效期有效且未启用可进行点击启用，需要有二次确认弹窗
                if (oldStatus == 2) {
                    if (rewardDO.getEndTime().toInstant().isBefore(Instant.now())) {
                        throw exception(REWARD_STATUS_NOT_ALLOW_ENABLE);
                    }
                    rewardMapper.updateById(RewardConvert.INSTANCE.convertStatusReqVO(statusVO));
                }
                break;
            default:
                throw exception(REWARD_STATUS_CHANGE_ERROR);
        }
    }

    //生成礼品ID
    private String generateRewardCode() {
        return RandomStringUtils.randomAlphanumeric(12).toUpperCase();
    }

    //校验礼品是否过期并修改礼品状态
    //TODO：异步更新礼品状态
    private void validateExpire(RewardDO rewardDO) {
        if (rewardDO.getStatus() == 1 && rewardDO.getEndTime().toInstant().isBefore(Instant.now())) {
            RewardDO expireReward = new RewardDO();
            expireReward.setId(rewardDO.getId());
            expireReward.setStatus(4);
            rewardMapper.updateById(expireReward);
        }
    }
}
